[data-md-color-accent=indigo] {
    --md-accent-fg-color: #5D9B40;
    --md-typeset-a-color: #5D9B40;
}

:root {
    --md-primary-fg-color: #071F2A;
    --md-primary-fg-color--light: #071F2A;
    --md-primary-fg-color--dark: #0e2833;
}

select {
    background-color: transparent;
    border: none;
    margin-left: 10px;
    padding: 5px;
    padding-right: 30px;
    /* Aumenta o padding à direita para dar espaço para a seta */
    font-size: 1em;
    color: inherit;
    /* Mantém a cor do texto consistente com o tema */
    appearance: none;
    /* Remove a aparência padrão do seletor no navegador */
    background-image: url('data:image/svg+xml;charset=UTF-8,%3Csvg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="%23fff" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-chevron-down"%3E%3Cpolyline points="6 9 12 15 18 9"%3E%3C/polyline%3E%3C/svg%3E');
    background-repeat: no-repeat;
    background-position: right 10px center;
    /* Ajusta a posição da seta */
    background-size: 1em;
    cursor: pointer;
}

select:focus {
    outline: none;
    /* Remove a borda de foco padrão */
}